-- LOGIC ANALYSER VIEWER
-- this file is only for simulation !!!!!!!!!
-- this file allows for viewing data obtained by internal logic analyzer
-- file la_data.bin should be on your computer
-- this file is obtained by a log_anal.vhd component and others components loaded into a FPGA (virtex)
-- and a proper WISHBONE reads
-- readblock  la_data.bin adr_start adr_stop
-- where adr_start= LA_base_address, adr_stop= LA_base_address + 2^(adr_width-1) + (15)dec
-- the last 16 bytes of la_data.bin contains control registers

-- adjust data_width and  mem_adr_width !!!! to be the same as in the log_anal entity !!!
-- also define your own signals (the same which where connected to log_anal data input
-- and then assign these signals to proper d(index) data

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity la_view is
 generic  -- should be the same as generics in the log_anal entity !!!
  ( constant RLC_max_count: integer:= 256; -- limit the max number of the same signal desplayed during simulation 
    constant trigger_same_as_data: boolean:= true; -- the captured data are the same as the trigger data, i.e. trig(trig_width-1 downto 0)= data(trig_width-1 downto 0) 
    constant file_name: string := "la_data.bin" -- file name which contains captured data
  );
end;

architecture la_view_arch of la_view is

function ReadGeneric(adr: integer) return integer is -- Read Generic values from data file
  variable ret, DataIn: integer;
  type BIT_VECTOR_FILE is file of integer;
  file Data_In_File : BIT_VECTOR_FILE;   
begin
  FILE_OPEN (Data_In_File, file_name, READ_MODE); 
	for i in 0 to adr/4 loop
	   READ (Data_In_File, DataIn);
	end loop;
  ret:= DataIn rem 256;
  assert not (adr<4 and ret/= 74)
	  report "Error in the file: Wrong file format - Logic Analyser identification byte 0 has different value than 0xA4"
 		severity failure; 
  ret:= (DataIn / (2**(8*(adr mod 4))));
  ret:= ret rem 256;
 	FILE_CLOSE (Data_In_File);
  return ret;    
end ReadGeneric;    
 
 constant ID: integer:= ReadGeneric(0); 
 constant data_width: integer:= 1 + ReadGeneric(2); -- width of the data that are analysed (must be power of 2)
 constant mem_adr_width: integer:= ReadGeneric(1) rem 32; -- internal memory address width  
 constant use_run_length_coding: integer:= (ReadGeneric(1) / 64) rem 2;
 constant trig_width: integer:= 1 + ReadGeneric(3); -- trigger data width
 constant ce_dwidth: integer:= 1 + ReadGeneric(4); -- clock enable data width
 constant file_length: integer:= 2**(mem_adr_width)*data_width/32; --  		-- number of dwords (32-bits) in the file (excluding control registers)
 constant mem_size: integer:= 2 ** mem_adr_width;
 type la_mem_type is array (mem_size-1 downto 0) of std_logic_vector (data_width-1 downto 0);
 constant ce_ctr: std_logic_vector(5 downto 0):= conv_std_logic_vector(ReadGeneric(10), 6);
 constant ce_ctr_ced_active_low: std_logic:= ce_ctr(0); -- ced logic is active low
 constant ce_ctr_ced_reg: std_logic:= ce_ctr(1); -- ced logic is registered
 constant ce_ctr_ce_dat: std_logic:= ce_ctr(2); -- capture only data that satisfy CE logic
 constant ce_ctr_and_trig: std_logic:= ce_ctr(3); -- trigger is active when trig logic and CE logic is met in the same time
 constant ce_ctr_or_trig: std_logic:= ce_ctr(4); -- trigger is active when trig logic OR CE logic is met
 constant ce_ctr_RLCn: std_logic:= ce_ctr(4); -- Run Length Coding is active (active low)

		  
procedure ReadControlReg(variable stop_count: out integer;
  signal trigger_value: out std_logic_vector(trig_width-1 downto 0);   -- read stop counter value from la_data.bin file
  signal ce_value: out std_logic_vector(ce_dwidth-1 downto 0)) is

  variable DataIn, stat: integer;
  variable trig_care: std_logic_vector(trig_width-1 downto 0);
  variable ce_care: std_logic_vector(ce_dwidth-1 downto 0); -- clock enable care signal
  type BIT_VECTOR_FILE is file of integer;
  file Data_In_File : BIT_VECTOR_FILE;   
begin
  FILE_OPEN (Data_In_File, file_name, READ_MODE); 
	for i in 0 to 2 loop -- read generic values
	   READ (Data_In_File, DataIn );
	end loop;
	-- Data_In contains status register (MSBs should be filled with zero)
  stat:= DataIn rem 256;
	assert stat<128
	  report "Warning. The log_anal was still in the run mode when data were read. Not all acquired data are valid"
	  severity warning;
	assert stat>=64
	  report "Warning. The log_anal has not finished data acquisition or file format error"
	  severity warning;
      
	READ (Data_In_File, DataIn ); --  stop counter
	assert DataIn< 2**mem_adr_width
	  report "Error. Incorect stop counter value in la_data.bin. Wrong input data format"
		  severity failure;
	stop_count:= DataIn;
	
	READ (Data_In_File, DataIn ); -- trig_value
  if trig_width<=32 then 
	  trigger_value<= conv_std_logic_vector(DataIn, trig_width);
  else
      trigger_value(31 downto 0)<= conv_std_logic_vector(DataIn, 32);
  end if;
	READ (Data_In_File, DataIn ); -- trig_care
  if trig_width<=32 then 
       trig_care:= conv_std_logic_vector(DataIn, trig_width);
  else
       trig_care(31 downto 0):= conv_std_logic_vector(DataIn, 32);
   end if; 
	 READ (Data_In_File, DataIn ); -- trig_value 64
   if trig_width> 32 then
	  trigger_value(trig_width-1 downto 32)<= conv_std_logic_vector(DataIn, trig_width-32);
   end if;      
	 READ (Data_In_File, DataIn ); -- trig_care 64\
   if trig_width> 32 then
	  trig_care(trig_width-1 downto 32):= conv_std_logic_vector(DataIn, trig_width-32);
   end if;      
   for i in trig_width-1 downto 0 loop
		if trig_care(i)='0' or 
            (i = data_width-1 and use_run_length_coding=1) then
	     trigger_value(i)<= '-';
		end if;
	end loop;

  --------- clock enable value  
  READ (Data_In_File, DataIn ); -- ce_value
  if ce_dwidth<=32 then 
	    ce_value<= conv_std_logic_vector(DataIn, ce_dwidth);
  else
      ce_value(31 downto 0)<= conv_std_logic_vector(DataIn, 32);
  end if;
	READ (Data_In_File, DataIn ); -- ce_care
  if ce_dwidth<=32 then 
       ce_care:= conv_std_logic_vector(DataIn, ce_dwidth);
  else
       ce_care(31 downto 0):= conv_std_logic_vector(DataIn, 32);
  end if; 
  READ (Data_In_File, DataIn ); -- ce_value64
  if ce_dwidth> 32 then
	  ce_value(ce_dwidth-1 downto 32)<= conv_std_logic_vector(DataIn, ce_dwidth-32);
  end if;
  READ (Data_In_File, DataIn ); -- ce_care64
	if ce_dwidth > 32 then
   ce_care(ce_dwidth-1 downto 32):= conv_std_logic_vector(DataIn, ce_dwidth-32);
  end if;      

  for i in ce_dwidth-1 downto 0 loop
		if ce_care(i)='0' or 
            (i = data_width-1 and use_run_length_coding=1) then
	     ce_value(i)<= '-';
		end if;
	end loop;

	FILE_CLOSE ( Data_In_File );
end ReadControlReg;
	


procedure ReadData(variable stop_count: in integer; signal mem : out la_mem_type ) is -- read recorded data from la_data.bin file
  variable DataIn : integer; 
  variable Data32 : std_logic_vector(31 downto 0);
  variable address:integer; 
  type BIT_VECTOR_FILE is file of integer;
  file DataInFile : BIT_VECTOR_FILE;
  variable read32: integer:= 0; -- for data_width=64 - which bytes: 0-31 or 32-64 are read

begin
	address:= (mem_size - stop_count) rem mem_size; -- the start address (rem is for stop_count=0)
  FILE_OPEN (DataInFile, file_name, READ_MODE); 
  for i in 1 to 12 loop -- read register data (not used in this procedure)
    READ(DataInFile, DataIn);
	end loop;
	for i in 1 to file_length loop -- for every data in the file
	  READ(DataInFile, DataIn);
	  Data32:= conv_std_logic_vector(DataIn, 32);
	  if data_width<=32 then 
	    for j in 0 to (32/data_width) -1 loop
		  mem(address)<= Data32((j+1)*data_width-1 downto j*data_width);
		  address:= address + 1;
		  if address= mem_size then
			  address:= 0; 
		  end if;
	    end loop;
	  else -- data width=64
		read32:= ((i-1) rem 2)*32; -- =0 the lower 4Bytes, 32- the upper 4Bytes
		mem(address)(31+read32 downto read32)<= Data32;
		if read32 = 32 then
			address:= address + 1;
			if address= mem_size then
			  address:= 0; 
		    end if;
		end if;
	  end if;		
	end loop;
end;
  -- find the data width which is redy=uced by 1 when use_run_length_codding>0
function FindDBusWidth(dwidth: integer) return integer is
begin
  if dwidth<=8 then return 1;
  elsif dwidth<=16 then return dwidth-8;
  elsif dwidth<=32 then return 8;
  else return 32;
  end if;
end FindDBusWidth;

  -- find the data width for ABus
function FindABusWidth(dwidth: integer) return integer is
begin
   if dwidth<= 16 then return 1;
   elsif dwidth<=32 then return dwidth-16;
   else return dwidth-40;
   end if;
end FindABusWidth;

  constant dwidth: integer:= data_width - use_run_length_coding;
  constant DBusWidth: integer:= FindDBusWidth(dwidth);
  constant ABusWidth: integer:= FindABusWidth(dwidth);

  signal la_mem : la_mem_type; -- data read form la_data.bin file
  signal d: std_logic_vector(dwidth-1 downto 0); -- data  recorder by the LA
  signal trigger_value: std_logic_vector(trig_width-1 downto 0); -- the value specified by writes to the trigger value and care registers
  signal trigger: std_logic; -- trigger condition is now satisfied
  signal ce_value: std_logic_vector(ce_dwidth-1 downto 0); -- clock enable (data are captured only when clock enable logic is satisfied
  signal clk: std_logic; -- data has been recoreded according to this clock activity
 
 -----------------------------------------------------------------------------------
 -- User area (changes can be done here)
 -- define here your signal names 
   signal OPB_busLock, OPB_seqAddr, OPB_RNW, OPB_errAck: std_logic;
   signal OPB_timeout, OPB_rst, OPB_xferAck, OPB_select: std_logic;
   signal OPB_DBus: std_logic_vector(0 to DBusWidth-1); 
   signal OPB_ABus: std_logic_vector(0 to ABusWidth-1); 

begin
  -- write here which signal is assigned to which LA data bus (see log_anal instantiation)
  -- signal 'd' is the same as data input in the log_anal entity
  OPB_select<= d(0);
  OPB_xferAck<= d(1);
  OPB_rst<= d(2);
  OPB_timeout<= d(3);
  OPB_errAck<= d(4);
  OPB_RNW<= d(5);
  OPB_seqAddr<= d(6);
  gg8: if dwidth>7 generate
    OPB_busLock<= d(7);
  end generate;
  gd16: if data_width= 16 or data_width=32  generate
   	OPB_DBus<= d(8 + DBusWidth -1 downto 8);
  end generate;
  
  gd32: if data_width=32 generate
	OPB_ABus<= d(dwidth-1 downto 8 + DBusWidth);
  end generate;

  gd64: if data_width=64 generate
    OPB_DBus<= d(39 downto 8);
	OPB_ABus<= d(dwidth-1 downto 40);
  end generate;
  
  
  -- clk generation
  process begin
	  wait for 10 ns; -- this value can be change freely
	  clk<= '0';
	  wait for 10 ns;
	  clk<= '1';
  end process;
  

-------------------------------------------------------------------------------------------
-- Internal LA logic do not change it !!!
process
  variable Initialize : integer := 0;	-- initialisation flag
  variable stop_count : integer; -- the count points where the last data has been written to the LA memory
begin
	if Initialize=0 then	-- run only one time at the beginning
	  ReadControlReg(stop_count, trigger_value, ce_value);
	  ReadData(stop_count, la_mem); -- read data recorded by the LA
	  Initialize := 1;
	  wait for 1000 ms;
    end if;
end process;


process(clk)
  variable i: integer:= 0;
  variable same_data_count: std_logic_vector(data_width-2 downto 0):= (others=>'0');
  variable zero: std_logic_vector(data_width-1 downto 0):= (others=>'0');
  variable trigger_tmp: std_logic;
  variable d_tmp: std_logic_vector(data_width-1 downto 0);
begin 
  if clk'event and clk='1' then

    if i= mem_size then
		  i:= mem_size-1;
		  assert false 
	        report "O.K. All acquired data in the LA has already been shown"
	        severity failure; 
	end if;

    if same_data_count/=zero then 
       same_data_count:= same_data_count - 1;
    else   
      d_tmp:= la_mem(i); -- data recorded in the LA
	  i:= i +1;
      if use_run_length_coding>0 and d_tmp(data_width-1)= '1' then -- same_data counter value
          same_data_count:= d_tmp(data_width-2 downto 0);
		  if same_data_count>RLC_max_count and RLC_max_count>=0 then
			  same_data_count:= conv_std_logic_vector(RLC_max_count, data_width-1);
		  end if;
      else
	    d<= d_tmp(dwidth-1 downto 0);
	    -- trigger logic
	    if trigger_same_as_data=true  and trig_width <= data_width then
		  trigger_tmp:= '1';
	      for j in trig_width-1 downto 0 loop
		    if trigger_value(j)/= '-' then -- check only if not don't care
			  trigger_tmp:= trigger_tmp AND not (trigger_value(j) XOR d_tmp(j));
		    end if;
	      end loop;	
	  	  trigger<= trigger_tmp;
	    else -- do not show trigger because trigger data are different from watched data
		  trigger<= 'Z';
	    end if;
	  end if; -- new data
    end if; --   same_data_count/=zero
  end if; --clk
end process;

assert data_width=64 or data_width=32 or data_width=16 or data_width=8 
	  report "Error in la_view: constant data_width must be 8, 16, 32 or 64"
 		severity failure; 

end la_view_arch;

-- logic analyser (LA) for FPGAs
-- ver 1.0
-- Author: Ernest Jamro

--//////////////////////////////////////////////////////////////////////
--//// Copyright (C) 2001 Authors and OPENCORES.ORG                 ////
--////                                                              ////
--//// This source file may be used and distributed without         ////
--/// restriction provided that this copyright statement is not    ////
--//// removed from the file and that any derivative work contains  ////
--//// the original copyright notice and the associated disclaimer. ////
--////                                                              ////
--//// This source file is free software; you can redistribute it   ////
--//// and/or modify it under the terms of the GNU Lesser General   ////
--//// Public License as published by the Free Software Foundation; ////
--//// either version 2.1 of the License, or (at your option) any   ////
--//// later version.                                               ////
--////                                                              ////
--//// This source is distributed in the hope that it will be       ////
--//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
--//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
--//// PURPOSE. See the GNU Lesser General Public License for more  ////
--//// details.                                                     ////
--////                                                              ////
--//// You should have received a copy of the GNU Lesser General    ////
--//// Public License along with this source; if not, download it   ////
--//// from <http://www.opencores.org/lgpl.shtml>                   ////
